#include "glow/IR/IRBuilder.h"
#include "glow/IR/IR.h"
#include "glow/IR/Instrs.h"

using namespace glow;

AllocActivationInst *IRBuilder::createAllocActivationInst(llvm::StringRef name, TypeRef Ty) {
  auto *A = new AllocActivationInst(uniqueName(name), Ty);
  F_->pushInstr(A);
  return A;
}

TensorViewInst *IRBuilder::createTensorViewInst(llvm::StringRef name, Value *Src, TypeRef Ty, std::vector<dim_t> Offsets) {
  auto *A = new TensorViewInst(uniqueName(name), Src, Ty, Offsets);
  F_->pushInstr(A);
  return A;
}

DeallocActivationInst *IRBuilder::createDeallocActivationInst(llvm::StringRef name, Value *Src) {
  auto *A = new DeallocActivationInst(uniqueName(name), Src);
  F_->pushInstr(A);
  return A;
}

CopyInst *IRBuilder::createCopyInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new CopyInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ConvolutionGradInst *IRBuilder::createConvolutionGradInst(llvm::StringRef name, Value *Src, Value *Filter, Value *DestGrad, Value *SrcGrad, Value *FilterGrad, Value *BiasGrad, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Group, unsigned_t Dilation, ConvolutionLayout Layout, FusedActivation FusedActivation) {
  auto *A = new ConvolutionGradInst(uniqueName(name), Src, Filter, DestGrad, SrcGrad, FilterGrad, BiasGrad, Kernels, Strides, Pads, Group, Dilation, Layout, FusedActivation);
  F_->pushInstr(A);
  return A;
}

ConvolutionInst *IRBuilder::createConvolutionInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Filter, Value *Bias, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Group, unsigned_t Dilation, ConvolutionLayout Layout, FusedActivation FusedActivation) {
  auto *A = new ConvolutionInst(uniqueName(name), Dest, Src, Filter, Bias, Kernels, Strides, Pads, Group, Dilation, Layout, FusedActivation);
  F_->pushInstr(A);
  return A;
}

ChannelwiseQuantizedConvolutionInst *IRBuilder::createChannelwiseQuantizedConvolutionInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Filter, Value *Bias, Value *FilterScales, Value *FilterOffsets, Value *BiasScales, Value *BiasOffsets, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Group, unsigned_t Dilation) {
  auto *A = new ChannelwiseQuantizedConvolutionInst(uniqueName(name), Dest, Src, Filter, Bias, FilterScales, FilterOffsets, BiasScales, BiasOffsets, Kernels, Strides, Pads, Group, Dilation);
  F_->pushInstr(A);
  return A;
}

ConvTransposeInst *IRBuilder::createConvTransposeInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Filter, Value *Bias, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Group, unsigned_t Dilation) {
  auto *A = new ConvTransposeInst(uniqueName(name), Dest, Src, Filter, Bias, Kernels, Strides, Pads, Group, Dilation);
  F_->pushInstr(A);
  return A;
}

Convolution3DGradInst *IRBuilder::createConvolution3DGradInst(llvm::StringRef name, Value *Src, Value *Filter, Value *DestGrad, Value *SrcGrad, Value *FilterGrad, Value *BiasGrad, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Group) {
  auto *A = new Convolution3DGradInst(uniqueName(name), Src, Filter, DestGrad, SrcGrad, FilterGrad, BiasGrad, Kernels, Strides, Pads, Group);
  F_->pushInstr(A);
  return A;
}

Convolution3DInst *IRBuilder::createConvolution3DInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Filter, Value *Bias, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Group) {
  auto *A = new Convolution3DInst(uniqueName(name), Dest, Src, Filter, Bias, Kernels, Strides, Pads, Group);
  F_->pushInstr(A);
  return A;
}

MaxPoolWithArgmaxGradInst *IRBuilder::createMaxPoolWithArgmaxGradInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Argmax, Value *DestGrad, Value *SrcGrad, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Layout) {
  auto *A = new MaxPoolWithArgmaxGradInst(uniqueName(name), Dest, Src, Argmax, DestGrad, SrcGrad, Kernels, Strides, Pads, Layout);
  F_->pushInstr(A);
  return A;
}

MaxPoolWithArgmaxInst *IRBuilder::createMaxPoolWithArgmaxInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Argmax, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Layout) {
  auto *A = new MaxPoolWithArgmaxInst(uniqueName(name), Dest, Src, Argmax, Kernels, Strides, Pads, Layout);
  F_->pushInstr(A);
  return A;
}

MaxPoolInst *IRBuilder::createMaxPoolInst(llvm::StringRef name, Value *Dest, Value *Src, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Layout) {
  auto *A = new MaxPoolInst(uniqueName(name), Dest, Src, Kernels, Strides, Pads, Layout);
  F_->pushInstr(A);
  return A;
}

AvgPoolGradInst *IRBuilder::createAvgPoolGradInst(llvm::StringRef name, Value *Dest, Value *Src, Value *DestGrad, Value *SrcGrad, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Layout) {
  auto *A = new AvgPoolGradInst(uniqueName(name), Dest, Src, DestGrad, SrcGrad, Kernels, Strides, Pads, Layout);
  F_->pushInstr(A);
  return A;
}

AvgPoolInst *IRBuilder::createAvgPoolInst(llvm::StringRef name, Value *Dest, Value *Src, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Layout) {
  auto *A = new AvgPoolInst(uniqueName(name), Dest, Src, Kernels, Strides, Pads, Layout);
  F_->pushInstr(A);
  return A;
}

ArgMaxInst *IRBuilder::createArgMaxInst(llvm::StringRef name, Value *Dest, Value *Src, unsigned_t Axis, bool KeepDims) {
  auto *A = new ArgMaxInst(uniqueName(name), Dest, Src, Axis, KeepDims);
  F_->pushInstr(A);
  return A;
}

ArgMinInst *IRBuilder::createArgMinInst(llvm::StringRef name, Value *Dest, Value *Src, unsigned_t Axis, bool KeepDims) {
  auto *A = new ArgMinInst(uniqueName(name), Dest, Src, Axis, KeepDims);
  F_->pushInstr(A);
  return A;
}

AdaptiveAvgPoolGradInst *IRBuilder::createAdaptiveAvgPoolGradInst(llvm::StringRef name, Value *Dest, Value *DestGrad, Value *SrcGrad) {
  auto *A = new AdaptiveAvgPoolGradInst(uniqueName(name), Dest, DestGrad, SrcGrad);
  F_->pushInstr(A);
  return A;
}

AdaptiveAvgPoolInst *IRBuilder::createAdaptiveAvgPoolInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new AdaptiveAvgPoolInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

FullyConnectedInst *IRBuilder::createFullyConnectedInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Weights, Value *Bias) {
  auto *A = new FullyConnectedInst(uniqueName(name), Dest, Src, Weights, Bias);
  F_->pushInstr(A);
  return A;
}

RowwiseQuantizedFullyConnectedInst *IRBuilder::createRowwiseQuantizedFullyConnectedInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Weights, Value *Bias, Value *Scales, Value *Offsets) {
  auto *A = new RowwiseQuantizedFullyConnectedInst(uniqueName(name), Dest, Src, Weights, Bias, Scales, Offsets);
  F_->pushInstr(A);
  return A;
}

LocalResponseNormalizationGradInst *IRBuilder::createLocalResponseNormalizationGradInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Scale, Value *DestGrad, Value *SrcGrad, unsigned_t HalfWindowSize, float Alpha, float Beta, float K) {
  auto *A = new LocalResponseNormalizationGradInst(uniqueName(name), Dest, Src, Scale, DestGrad, SrcGrad, HalfWindowSize, Alpha, Beta, K);
  F_->pushInstr(A);
  return A;
}

LocalResponseNormalizationInst *IRBuilder::createLocalResponseNormalizationInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Scale, unsigned_t HalfWindowSize, float Alpha, float Beta, float K) {
  auto *A = new LocalResponseNormalizationInst(uniqueName(name), Dest, Src, Scale, HalfWindowSize, Alpha, Beta, K);
  F_->pushInstr(A);
  return A;
}

SoftMaxInst *IRBuilder::createSoftMaxInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new SoftMaxInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

SoftMaxGradInst *IRBuilder::createSoftMaxGradInst(llvm::StringRef name, Value *OrigDest, Value *OrigSrc, Value *Selected, Value *SrcGrad) {
  auto *A = new SoftMaxGradInst(uniqueName(name), OrigDest, OrigSrc, Selected, SrcGrad);
  F_->pushInstr(A);
  return A;
}

CrossEntropyLossInst *IRBuilder::createCrossEntropyLossInst(llvm::StringRef name, Value *P, Value *Labels, Value *CE) {
  auto *A = new CrossEntropyLossInst(uniqueName(name), P, Labels, CE);
  F_->pushInstr(A);
  return A;
}

CrossEntropyLossGradInst *IRBuilder::createCrossEntropyLossGradInst(llvm::StringRef name, Value *CEGrad, Value *P, Value *Labels, Value *Pgrad, Value *Labelsgrad) {
  auto *A = new CrossEntropyLossGradInst(uniqueName(name), CEGrad, P, Labels, Pgrad, Labelsgrad);
  F_->pushInstr(A);
  return A;
}

MatMulInst *IRBuilder::createMatMulInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new MatMulInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

BatchMatMulInst *IRBuilder::createBatchMatMulInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new BatchMatMulInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

BatchedReduceAddInst *IRBuilder::createBatchedReduceAddInst(llvm::StringRef name, Value *Dest, Value *Batch, unsigned_t Axis) {
  auto *A = new BatchedReduceAddInst(uniqueName(name), Dest, Batch, Axis);
  F_->pushInstr(A);
  return A;
}

BatchedReduceMinInst *IRBuilder::createBatchedReduceMinInst(llvm::StringRef name, Value *Dest, Value *Batch, std::vector<unsigned_t> Axes) {
  auto *A = new BatchedReduceMinInst(uniqueName(name), Dest, Batch, Axes);
  F_->pushInstr(A);
  return A;
}

CumSumInst *IRBuilder::createCumSumInst(llvm::StringRef name, Value *Dest, Value *Input, unsigned_t Exclusive, unsigned_t Reverse) {
  auto *A = new CumSumInst(uniqueName(name), Dest, Input, Exclusive, Reverse);
  F_->pushInstr(A);
  return A;
}

LengthsSumInst *IRBuilder::createLengthsSumInst(llvm::StringRef name, Value *Dest, Value *Data, Value *Lengths) {
  auto *A = new LengthsSumInst(uniqueName(name), Dest, Data, Lengths);
  F_->pushInstr(A);
  return A;
}

SparseLengthsSumGradInst *IRBuilder::createSparseLengthsSumGradInst(llvm::StringRef name, Value *Data, Value *Indices, Value *Lengths, Value *DestGrad, Value *DataGrad, glow::LengthsMode LengthsMode, float AvgLength) {
  auto *A = new SparseLengthsSumGradInst(uniqueName(name), Data, Indices, Lengths, DestGrad, DataGrad, LengthsMode, AvgLength);
  F_->pushInstr(A);
  return A;
}

SparseLengthsSumInst *IRBuilder::createSparseLengthsSumInst(llvm::StringRef name, Value *Dest, Value *Data, Value *Indices, Value *Lengths, glow::LengthsMode LengthsMode, float AvgLength) {
  auto *A = new SparseLengthsSumInst(uniqueName(name), Dest, Data, Indices, Lengths, LengthsMode, AvgLength);
  F_->pushInstr(A);
  return A;
}

SparseLengthsWeightedSumGradInst *IRBuilder::createSparseLengthsWeightedSumGradInst(llvm::StringRef name, Value *Data, Value *Weights, Value *Indices, Value *Lengths, Value *DestGrad, Value *DataGrad, Value *WeightsGrad, glow::LengthsMode LengthsMode, float AvgLength) {
  auto *A = new SparseLengthsWeightedSumGradInst(uniqueName(name), Data, Weights, Indices, Lengths, DestGrad, DataGrad, WeightsGrad, LengthsMode, AvgLength);
  F_->pushInstr(A);
  return A;
}

SparseLengthsWeightedSumInst *IRBuilder::createSparseLengthsWeightedSumInst(llvm::StringRef name, Value *Dest, Value *Data, Value *Weights, Value *Indices, Value *Lengths, glow::LengthsMode LengthsMode, float AvgLength) {
  auto *A = new SparseLengthsWeightedSumInst(uniqueName(name), Dest, Data, Weights, Indices, Lengths, LengthsMode, AvgLength);
  F_->pushInstr(A);
  return A;
}

EmbeddingBagInst *IRBuilder::createEmbeddingBagInst(llvm::StringRef name, Value *Dest, Value *Data, Value *Weights, Value *Indices, Value *Offsets, bool HasEndOffset, glow::LengthsMode LengthsMode, float AvgLength) {
  auto *A = new EmbeddingBagInst(uniqueName(name), Dest, Data, Weights, Indices, Offsets, HasEndOffset, LengthsMode, AvgLength);
  F_->pushInstr(A);
  return A;
}

RowwiseQuantizedSparseLengthsWeightedSumInst *IRBuilder::createRowwiseQuantizedSparseLengthsWeightedSumInst(llvm::StringRef name, Value *Dest, Value *Data, Value *Scales, Value *Offsets, Value *Weights, Value *Indices, Value *Lengths, bool UseFP16Accumulation, glow::LengthsMode LengthsMode, float AvgLength) {
  auto *A = new RowwiseQuantizedSparseLengthsWeightedSumInst(uniqueName(name), Dest, Data, Scales, Offsets, Weights, Indices, Lengths, UseFP16Accumulation, LengthsMode, AvgLength);
  F_->pushInstr(A);
  return A;
}

FusedRowwiseQuantizedSparseLengthsWeightedSumInst *IRBuilder::createFusedRowwiseQuantizedSparseLengthsWeightedSumInst(llvm::StringRef name, Value *Dest, Value *Data, Value *Weights, Value *Indices, Value *Lengths, bool UseFP16Accumulation, glow::LengthsMode LengthsMode, float AvgLength) {
  auto *A = new FusedRowwiseQuantizedSparseLengthsWeightedSumInst(uniqueName(name), Dest, Data, Weights, Indices, Lengths, UseFP16Accumulation, LengthsMode, AvgLength);
  F_->pushInstr(A);
  return A;
}

EmbeddingBagByteRowwiseOffsetsInst *IRBuilder::createEmbeddingBagByteRowwiseOffsetsInst(llvm::StringRef name, Value *Dest, Value *Data, Value *Weights, Value *Indices, Value *Offsets, bool UseFP16Accumulation, bool HasEndOffset, glow::LengthsMode LengthsMode, float AvgLength) {
  auto *A = new EmbeddingBagByteRowwiseOffsetsInst(uniqueName(name), Dest, Data, Weights, Indices, Offsets, UseFP16Accumulation, HasEndOffset, LengthsMode, AvgLength);
  F_->pushInstr(A);
  return A;
}

LengthsToRangesInst *IRBuilder::createLengthsToRangesInst(llvm::StringRef name, Value *Dest, Value *Lengths) {
  auto *A = new LengthsToRangesInst(uniqueName(name), Dest, Lengths);
  F_->pushInstr(A);
  return A;
}

LengthsRangeFillInst *IRBuilder::createLengthsRangeFillInst(llvm::StringRef name, Value *Dest, Value *Lengths) {
  auto *A = new LengthsRangeFillInst(uniqueName(name), Dest, Lengths);
  F_->pushInstr(A);
  return A;
}

SparseToDenseInst *IRBuilder::createSparseToDenseInst(llvm::StringRef name, Value *Dest, Value *Indices, Value *Values) {
  auto *A = new SparseToDenseInst(uniqueName(name), Dest, Indices, Values);
  F_->pushInstr(A);
  return A;
}

SparseToDenseMaskInst *IRBuilder::createSparseToDenseMaskInst(llvm::StringRef name, Value *Dest, Value *Indices, Value *Values, Value *DefaultValue, Value *Lengths, std::vector<dim_t> Mask) {
  auto *A = new SparseToDenseMaskInst(uniqueName(name), Dest, Indices, Values, DefaultValue, Lengths, Mask);
  F_->pushInstr(A);
  return A;
}

BatchedAddInst *IRBuilder::createBatchedAddInst(llvm::StringRef name, Value *Dest, Value *Batch, Value *Slice) {
  auto *A = new BatchedAddInst(uniqueName(name), Dest, Batch, Slice);
  F_->pushInstr(A);
  return A;
}

ElementAddInst *IRBuilder::createElementAddInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementAddInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementSubInst *IRBuilder::createElementSubInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementSubInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementMulInst *IRBuilder::createElementMulInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementMulInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementDivInst *IRBuilder::createElementDivInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementDivInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementMaxInst *IRBuilder::createElementMaxInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementMaxInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementMinInst *IRBuilder::createElementMinInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementMinInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementCmpEQInst *IRBuilder::createElementCmpEQInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementCmpEQInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementCmpNEQInst *IRBuilder::createElementCmpNEQInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementCmpNEQInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementCmpLTInst *IRBuilder::createElementCmpLTInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementCmpLTInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementCmpLTEInst *IRBuilder::createElementCmpLTEInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementCmpLTEInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementIsNaNInst *IRBuilder::createElementIsNaNInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementIsNaNInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementPowInst *IRBuilder::createElementPowInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementPowInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementAndInst *IRBuilder::createElementAndInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementAndInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementOrInst *IRBuilder::createElementOrInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementOrInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementXorInst *IRBuilder::createElementXorInst(llvm::StringRef name, Value *Dest, Value *LHS, Value *RHS) {
  auto *A = new ElementXorInst(uniqueName(name), Dest, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ElementNotInst *IRBuilder::createElementNotInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementNotInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementNegInst *IRBuilder::createElementNegInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementNegInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementAbsInst *IRBuilder::createElementAbsInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementAbsInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementFloorInst *IRBuilder::createElementFloorInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementFloorInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementCeilInst *IRBuilder::createElementCeilInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementCeilInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementRoundInst *IRBuilder::createElementRoundInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementRoundInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementSqrtInst *IRBuilder::createElementSqrtInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementSqrtInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementRsqrtInst *IRBuilder::createElementRsqrtInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementRsqrtInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementReciprocalInst *IRBuilder::createElementReciprocalInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementReciprocalInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementSinInst *IRBuilder::createElementSinInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementSinInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementCosInst *IRBuilder::createElementCosInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementCosInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementLogInst *IRBuilder::createElementLogInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementLogInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementExpInst *IRBuilder::createElementExpInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ElementExpInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ElementSelectInst *IRBuilder::createElementSelectInst(llvm::StringRef name, Value *Dest, Value *Cond, Value *LHS, Value *RHS) {
  auto *A = new ElementSelectInst(uniqueName(name), Dest, Cond, LHS, RHS);
  F_->pushInstr(A);
  return A;
}

ModuloInst *IRBuilder::createModuloInst(llvm::StringRef name, Value *Dest, Value *Src, int64_t Divisor, bool SignFollowDivisor) {
  auto *A = new ModuloInst(uniqueName(name), Dest, Src, Divisor, SignFollowDivisor);
  F_->pushInstr(A);
  return A;
}

BatchedPairwiseDotProductInst *IRBuilder::createBatchedPairwiseDotProductInst(llvm::StringRef name, Value *Dest, unsigned_t NumInputs, unsigned_t VectorSize) {
  auto *A = new BatchedPairwiseDotProductInst(uniqueName(name), Dest, NumInputs, VectorSize);
  F_->pushInstr(A);
  return A;
}

BatchedPairwiseDotProductGradInst *IRBuilder::createBatchedPairwiseDotProductGradInst(llvm::StringRef name, Value *DestGrad, unsigned_t NumInputs, unsigned_t VectorSize) {
  auto *A = new BatchedPairwiseDotProductGradInst(uniqueName(name), DestGrad, NumInputs, VectorSize);
  F_->pushInstr(A);
  return A;
}

ReluGradInst *IRBuilder::createReluGradInst(llvm::StringRef name, Value *Dest, Value *DestGrad, Value *SrcGrad) {
  auto *A = new ReluGradInst(uniqueName(name), Dest, DestGrad, SrcGrad);
  F_->pushInstr(A);
  return A;
}

ReluInst *IRBuilder::createReluInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new ReluInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

ClipInst *IRBuilder::createClipInst(llvm::StringRef name, Value *Dest, Value *Src, float Min, float Max) {
  auto *A = new ClipInst(uniqueName(name), Dest, Src, Min, Max);
  F_->pushInstr(A);
  return A;
}

SigmoidInst *IRBuilder::createSigmoidInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new SigmoidInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

TanhInst *IRBuilder::createTanhInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new TanhInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

TransposeInst *IRBuilder::createTransposeInst(llvm::StringRef name, Value *Dest, Value *Src, std::vector<unsigned_t> Shuffle) {
  auto *A = new TransposeInst(uniqueName(name), Dest, Src, Shuffle);
  F_->pushInstr(A);
  return A;
}

SplatInst *IRBuilder::createSplatInst(llvm::StringRef name, Value *Dest, float Value) {
  auto *A = new SplatInst(uniqueName(name), Dest, Value);
  F_->pushInstr(A);
  return A;
}

TouchInst *IRBuilder::createTouchInst(llvm::StringRef name, Value *Dest) {
  auto *A = new TouchInst(uniqueName(name), Dest);
  F_->pushInstr(A);
  return A;
}

InsertTensorInst *IRBuilder::createInsertTensorInst(llvm::StringRef name, Value *Dest, Value *Src, std::vector<dim_t> Offsets, unsigned_t Count, unsigned_t Axis) {
  auto *A = new InsertTensorInst(uniqueName(name), Dest, Src, Offsets, Count, Axis);
  F_->pushInstr(A);
  return A;
}

ExtractTensorInst *IRBuilder::createExtractTensorInst(llvm::StringRef name, Value *Dest, Value *Src, std::vector<dim_t> Offsets) {
  auto *A = new ExtractTensorInst(uniqueName(name), Dest, Src, Offsets);
  F_->pushInstr(A);
  return A;
}

GatherInst *IRBuilder::createGatherInst(llvm::StringRef name, Value *Dest, Value *Data, Value *Indices, unsigned_t BatchDims) {
  auto *A = new GatherInst(uniqueName(name), Dest, Data, Indices, BatchDims);
  F_->pushInstr(A);
  return A;
}

GatherRangesInst *IRBuilder::createGatherRangesInst(llvm::StringRef name, Value *Output, Value *Lengths, Value *Data, Value *Ranges) {
  auto *A = new GatherRangesInst(uniqueName(name), Output, Lengths, Data, Ranges);
  F_->pushInstr(A);
  return A;
}

ScatterDataInst *IRBuilder::createScatterDataInst(llvm::StringRef name, Value *Data, Value *Indices, Value *Slices, bool Cumulative) {
  auto *A = new ScatterDataInst(uniqueName(name), Data, Indices, Slices, Cumulative);
  F_->pushInstr(A);
  return A;
}

BatchOneHotInst *IRBuilder::createBatchOneHotInst(llvm::StringRef name, Value *Dest, Value *Data, Value *Lengths, Value *Values) {
  auto *A = new BatchOneHotInst(uniqueName(name), Dest, Data, Lengths, Values);
  F_->pushInstr(A);
  return A;
}

SpaceToDepthInst *IRBuilder::createSpaceToDepthInst(llvm::StringRef name, Value *Dest, Value *Src, unsigned_t BlockSize) {
  auto *A = new SpaceToDepthInst(uniqueName(name), Dest, Src, BlockSize);
  F_->pushInstr(A);
  return A;
}

ResizeNearestInst *IRBuilder::createResizeNearestInst(llvm::StringRef name, Value *Dest, Value *Src, std::vector<float> Scale) {
  auto *A = new ResizeNearestInst(uniqueName(name), Dest, Src, Scale);
  F_->pushInstr(A);
  return A;
}

ResizeBilinearInst *IRBuilder::createResizeBilinearInst(llvm::StringRef name, Value *Dest, Value *Src, std::vector<float> Scale) {
  auto *A = new ResizeBilinearInst(uniqueName(name), Dest, Src, Scale);
  F_->pushInstr(A);
  return A;
}

FlipInst *IRBuilder::createFlipInst(llvm::StringRef name, Value *Dest, Value *Src, unsigned_t Axis) {
  auto *A = new FlipInst(uniqueName(name), Dest, Src, Axis);
  F_->pushInstr(A);
  return A;
}

DebugPrintInst *IRBuilder::createDebugPrintInst(llvm::StringRef name, Value *Src, std::string Format, std::string FileName) {
  auto *A = new DebugPrintInst(uniqueName(name), Src, Format, FileName);
  F_->pushInstr(A);
  return A;
}

TraceEventInst *IRBuilder::createTraceEventInst(llvm::StringRef name, Value *Data, unsigned_t Index) {
  auto *A = new TraceEventInst(uniqueName(name), Data, Index);
  F_->pushInstr(A);
  return A;
}

QuantizationProfileInst *IRBuilder::createQuantizationProfileInst(llvm::StringRef name, Value *InputTensor, Value *Histogram, Value *ComputationInfo) {
  auto *A = new QuantizationProfileInst(uniqueName(name), InputTensor, Histogram, ComputationInfo);
  F_->pushInstr(A);
  return A;
}

IntLookupTableInst *IRBuilder::createIntLookupTableInst(llvm::StringRef name, Value *Dest, Value *Src, Value *Mapping) {
  auto *A = new IntLookupTableInst(uniqueName(name), Dest, Src, Mapping);
  F_->pushInstr(A);
  return A;
}

QuantizeInst *IRBuilder::createQuantizeInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new QuantizeInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

DequantizeInst *IRBuilder::createDequantizeInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new DequantizeInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

RescaleQuantizedInst *IRBuilder::createRescaleQuantizedInst(llvm::StringRef name, Value *Dest, Value *Src) {
  auto *A = new RescaleQuantizedInst(uniqueName(name), Dest, Src);
  F_->pushInstr(A);
  return A;
}

TopKInst *IRBuilder::createTopKInst(llvm::StringRef name, Value *Values, Value *Indices, Value *Input, unsigned_t K) {
  std::string ScratchName = name.str() + ".scratch";
  auto *ScratchType = F_->getGraph()->getParent()->uniqueType(ElemKind::Int8QTy, {1}, 0.0, 0);
  auto *Scratch = createAllocActivationInst(ScratchName, ScratchType);
  auto *A = new TopKInst(uniqueName(name), Values, Indices, Input, Scratch, K);
  dim_t ScratchSizeVar = static_cast<dim_t>(A->getScratchSize());
  ScratchSizeVar = ScratchSizeVar > 0 ? ScratchSizeVar : 1;
  auto *ScratchTypeResized = F_->getGraph()->getParent()->uniqueType(ElemKind::Int8QTy, {ScratchSizeVar}, 0.0, 0);
  Scratch->setType(ScratchTypeResized);
  Scratch->setTy(ScratchTypeResized);
  F_->pushInstr(A);
  return A;
}

ConvertToInst *IRBuilder::createConvertToInst(llvm::StringRef name, Value *Result, Value *Input) {
  auto *A = new ConvertToInst(uniqueName(name), Result, Input);
  F_->pushInstr(A);
  return A;
}

AudioSpectrogramInst *IRBuilder::createAudioSpectrogramInst(llvm::StringRef name, Value *Spectrogram, Value *Input, Value *Window, Value *TwiddleFactors, Value *BitReverseIndices, Value *ComplexToRealWeights, int64_t WindowSize, int64_t WindowStride, bool MagnitudeSquared) {
  std::string WinOutScratchName = name.str() + ".winoutscratch";
  auto *WinOutScratchType = F_->getGraph()->getParent()->uniqueType(ElemKind::Int8QTy, {1}, 0.0, 0);
  auto *WinOutScratch = createAllocActivationInst(WinOutScratchName, WinOutScratchType);
  std::string FftOutScratchName = name.str() + ".fftoutscratch";
  auto *FftOutScratchType = F_->getGraph()->getParent()->uniqueType(ElemKind::Int8QTy, {1}, 0.0, 0);
  auto *FftOutScratch = createAllocActivationInst(FftOutScratchName, FftOutScratchType);
  auto *A = new AudioSpectrogramInst(uniqueName(name), Spectrogram, Input, Window, TwiddleFactors, BitReverseIndices, ComplexToRealWeights, WinOutScratch, FftOutScratch, WindowSize, WindowStride, MagnitudeSquared);
  dim_t WinOutScratchSizeVar = static_cast<dim_t>(A->getWinOutScratchSize());
  WinOutScratchSizeVar = WinOutScratchSizeVar > 0 ? WinOutScratchSizeVar : 1;
  auto *WinOutScratchTypeResized = F_->getGraph()->getParent()->uniqueType(ElemKind::Int8QTy, {WinOutScratchSizeVar}, 0.0, 0);
  WinOutScratch->setType(WinOutScratchTypeResized);
  WinOutScratch->setTy(WinOutScratchTypeResized);
  dim_t FftOutScratchSizeVar = static_cast<dim_t>(A->getFftOutScratchSize());
  FftOutScratchSizeVar = FftOutScratchSizeVar > 0 ? FftOutScratchSizeVar : 1;
  auto *FftOutScratchTypeResized = F_->getGraph()->getParent()->uniqueType(ElemKind::Int8QTy, {FftOutScratchSizeVar}, 0.0, 0);
  FftOutScratch->setType(FftOutScratchTypeResized);
  FftOutScratch->setTy(FftOutScratchTypeResized);
  F_->pushInstr(A);
  return A;
}

MFCCInst *IRBuilder::createMFCCInst(llvm::StringRef name, Value *Coefficients, Value *Spectrogram, Value *MelWeights, Value *MelRanges, Value *DctMat, float SampleRate, float LowerFrequency, float UpperFrequency, int64_t FilterBankCount, int64_t NumCoefficients) {
  std::string ScratchName = name.str() + ".scratch";
  auto *ScratchType = F_->getGraph()->getParent()->uniqueType(ElemKind::Int8QTy, {1}, 0.0, 0);
  auto *Scratch = createAllocActivationInst(ScratchName, ScratchType);
  auto *A = new MFCCInst(uniqueName(name), Coefficients, Spectrogram, MelWeights, MelRanges, DctMat, Scratch, SampleRate, LowerFrequency, UpperFrequency, FilterBankCount, NumCoefficients);
  dim_t ScratchSizeVar = static_cast<dim_t>(A->getScratchSize());
  ScratchSizeVar = ScratchSizeVar > 0 ? ScratchSizeVar : 1;
  auto *ScratchTypeResized = F_->getGraph()->getParent()->uniqueType(ElemKind::Int8QTy, {ScratchSizeVar}, 0.0, 0);
  Scratch->setType(ScratchTypeResized);
  Scratch->setTy(ScratchTypeResized);
  F_->pushInstr(A);
  return A;
}

NonMaxSuppressionInst *IRBuilder::createNonMaxSuppressionInst(llvm::StringRef name, Value *Indices, Value *NumberOfSelectedIndices, Value *Boxes, Value *Scores, int64_t CenterPointBox, int64_t MaxOutputBoxesPerClass, float IouThreshold, float ScoreThreshold, bool IsTFVersion4) {
  auto *A = new NonMaxSuppressionInst(uniqueName(name), Indices, NumberOfSelectedIndices, Boxes, Scores, CenterPointBox, MaxOutputBoxesPerClass, IouThreshold, ScoreThreshold, IsTFVersion4);
  F_->pushInstr(A);
  return A;
}

CPUMaxSplatInst *IRBuilder::createCPUMaxSplatInst(llvm::StringRef name, Value *Dest, Value *Src, float SplatValue) {
  auto *A = new CPUMaxSplatInst(uniqueName(name), Dest, Src, SplatValue);
  F_->pushInstr(A);
  return A;
}

CPUConvDKKC8Inst *IRBuilder::createCPUConvDKKC8Inst(llvm::StringRef name, Value *Dest, Value *Src, Value *Filter, Value *Bias, std::vector<unsigned_t> Kernels, std::vector<unsigned_t> Strides, std::vector<unsigned_t> Pads, unsigned_t Group) {
  auto *A = new CPUConvDKKC8Inst(uniqueName(name), Dest, Src, Filter, Bias, Kernels, Strides, Pads, Group);
  F_->pushInstr(A);
  return A;
}
