#include "glow/IR/IR.h"
#include "glow/IR/Instrs.h"
#include "glow/Base/Type.h"
#include "glow/Support/Support.h"

using namespace glow;

void AllocActivationInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Ty: " << getTy()
     << "}";
}

Instruction* AllocActivationInst::clone() const {
  return new AllocActivationInst(getName(), getTy());
}

llvm::StringRef AllocActivationInst::getOperandName(unsigned idx) const {
  llvm_unreachable("Invalid index");
}

void AllocActivationInst::setTy(TypeRef Ty) { Ty_ = Ty; }

void TensorViewInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Ty: " << getTy()
     << ", Offsets: " << getOffsets()
     << "}";
}

Instruction* TensorViewInst::clone() const {
  return new TensorViewInst(getName(), getSrc(), getTy(), getOffsets());
}

llvm::StringRef TensorViewInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void DeallocActivationInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* DeallocActivationInst::clone() const {
  return new DeallocActivationInst(getName(), getSrc());
}

llvm::StringRef DeallocActivationInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Src"; }
  llvm_unreachable("Invalid index");
}

AllocActivationInst *DeallocActivationInst::getAlloc() const { return  llvm::cast<AllocActivationInst>(getSrc()); }

void CopyInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* CopyInst::clone() const {
  return new CopyInst(getName(), getDest(), getSrc());
}

llvm::StringRef CopyInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ConvolutionGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << ", Dilation: " << getDilation()
     << ", Layout: " << getLayout()
     << ", FusedActivation: " << getFusedActivation()
     << "}";
}

Instruction* ConvolutionGradInst::clone() const {
  return new ConvolutionGradInst(getName(), getSrc(), getFilter(), getDestGrad(), getSrcGrad(), getFilterGrad(), getBiasGrad(), getKernels(), getStrides(), getPads(), getGroup(), getDilation(), getLayout(), getFusedActivation());
}

llvm::StringRef ConvolutionGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Src"; }
  if (idx == 1) { return "Filter"; }
  if (idx == 2) { return "DestGrad"; }
  if (idx == 3) { return "SrcGrad"; }
  if (idx == 4) { return "FilterGrad"; }
  if (idx == 5) { return "BiasGrad"; }
  llvm_unreachable("Invalid index");
}

void ConvolutionInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << ", Dilation: " << getDilation()
     << ", Layout: " << getLayout()
     << ", FusedActivation: " << getFusedActivation()
     << "}";
}

Instruction* ConvolutionInst::clone() const {
  return new ConvolutionInst(getName(), getDest(), getSrc(), getFilter(), getBias(), getKernels(), getStrides(), getPads(), getGroup(), getDilation(), getLayout(), getFusedActivation());
}

llvm::StringRef ConvolutionInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Filter"; }
  if (idx == 3) { return "Bias"; }
  llvm_unreachable("Invalid index");
}

void ChannelwiseQuantizedConvolutionInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << ", Dilation: " << getDilation()
     << "}";
}

Instruction* ChannelwiseQuantizedConvolutionInst::clone() const {
  return new ChannelwiseQuantizedConvolutionInst(getName(), getDest(), getSrc(), getFilter(), getBias(), getFilterScales(), getFilterOffsets(), getBiasScales(), getBiasOffsets(), getKernels(), getStrides(), getPads(), getGroup(), getDilation());
}

llvm::StringRef ChannelwiseQuantizedConvolutionInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Filter"; }
  if (idx == 3) { return "Bias"; }
  if (idx == 4) { return "FilterScales"; }
  if (idx == 5) { return "FilterOffsets"; }
  if (idx == 6) { return "BiasScales"; }
  if (idx == 7) { return "BiasOffsets"; }
  llvm_unreachable("Invalid index");
}

void ConvTransposeInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << ", Dilation: " << getDilation()
     << "}";
}

Instruction* ConvTransposeInst::clone() const {
  return new ConvTransposeInst(getName(), getDest(), getSrc(), getFilter(), getBias(), getKernels(), getStrides(), getPads(), getGroup(), getDilation());
}

llvm::StringRef ConvTransposeInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Filter"; }
  if (idx == 3) { return "Bias"; }
  llvm_unreachable("Invalid index");
}

void Convolution3DGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << "}";
}

Instruction* Convolution3DGradInst::clone() const {
  return new Convolution3DGradInst(getName(), getSrc(), getFilter(), getDestGrad(), getSrcGrad(), getFilterGrad(), getBiasGrad(), getKernels(), getStrides(), getPads(), getGroup());
}

llvm::StringRef Convolution3DGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Src"; }
  if (idx == 1) { return "Filter"; }
  if (idx == 2) { return "DestGrad"; }
  if (idx == 3) { return "SrcGrad"; }
  if (idx == 4) { return "FilterGrad"; }
  if (idx == 5) { return "BiasGrad"; }
  llvm_unreachable("Invalid index");
}

void Convolution3DInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << "}";
}

Instruction* Convolution3DInst::clone() const {
  return new Convolution3DInst(getName(), getDest(), getSrc(), getFilter(), getBias(), getKernels(), getStrides(), getPads(), getGroup());
}

llvm::StringRef Convolution3DInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Filter"; }
  if (idx == 3) { return "Bias"; }
  llvm_unreachable("Invalid index");
}

void MaxPoolWithArgmaxGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

Instruction* MaxPoolWithArgmaxGradInst::clone() const {
  return new MaxPoolWithArgmaxGradInst(getName(), getDest(), getSrc(), getArgmax(), getDestGrad(), getSrcGrad(), getKernels(), getStrides(), getPads(), getLayout());
}

llvm::StringRef MaxPoolWithArgmaxGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Argmax"; }
  if (idx == 3) { return "DestGrad"; }
  if (idx == 4) { return "SrcGrad"; }
  llvm_unreachable("Invalid index");
}

void MaxPoolWithArgmaxInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

Instruction* MaxPoolWithArgmaxInst::clone() const {
  return new MaxPoolWithArgmaxInst(getName(), getDest(), getSrc(), getArgmax(), getKernels(), getStrides(), getPads(), getLayout());
}

llvm::StringRef MaxPoolWithArgmaxInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Argmax"; }
  llvm_unreachable("Invalid index");
}

void MaxPoolInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

Instruction* MaxPoolInst::clone() const {
  return new MaxPoolInst(getName(), getDest(), getSrc(), getKernels(), getStrides(), getPads(), getLayout());
}

llvm::StringRef MaxPoolInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void AvgPoolGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

Instruction* AvgPoolGradInst::clone() const {
  return new AvgPoolGradInst(getName(), getDest(), getSrc(), getDestGrad(), getSrcGrad(), getKernels(), getStrides(), getPads(), getLayout());
}

llvm::StringRef AvgPoolGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "DestGrad"; }
  if (idx == 3) { return "SrcGrad"; }
  llvm_unreachable("Invalid index");
}

void AvgPoolInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

Instruction* AvgPoolInst::clone() const {
  return new AvgPoolInst(getName(), getDest(), getSrc(), getKernels(), getStrides(), getPads(), getLayout());
}

llvm::StringRef AvgPoolInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ArgMaxInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Axis: " << getAxis()
     << ", KeepDims: " << getKeepDims()
     << "}";
}

Instruction* ArgMaxInst::clone() const {
  return new ArgMaxInst(getName(), getDest(), getSrc(), getAxis(), getKeepDims());
}

llvm::StringRef ArgMaxInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ArgMinInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Axis: " << getAxis()
     << ", KeepDims: " << getKeepDims()
     << "}";
}

Instruction* ArgMinInst::clone() const {
  return new ArgMinInst(getName(), getDest(), getSrc(), getAxis(), getKeepDims());
}

llvm::StringRef ArgMinInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void AdaptiveAvgPoolGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* AdaptiveAvgPoolGradInst::clone() const {
  return new AdaptiveAvgPoolGradInst(getName(), getDest(), getDestGrad(), getSrcGrad());
}

llvm::StringRef AdaptiveAvgPoolGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "DestGrad"; }
  if (idx == 2) { return "SrcGrad"; }
  llvm_unreachable("Invalid index");
}

void AdaptiveAvgPoolInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* AdaptiveAvgPoolInst::clone() const {
  return new AdaptiveAvgPoolInst(getName(), getDest(), getSrc());
}

llvm::StringRef AdaptiveAvgPoolInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void FullyConnectedInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* FullyConnectedInst::clone() const {
  return new FullyConnectedInst(getName(), getDest(), getSrc(), getWeights(), getBias());
}

llvm::StringRef FullyConnectedInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Weights"; }
  if (idx == 3) { return "Bias"; }
  llvm_unreachable("Invalid index");
}

void RowwiseQuantizedFullyConnectedInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* RowwiseQuantizedFullyConnectedInst::clone() const {
  return new RowwiseQuantizedFullyConnectedInst(getName(), getDest(), getSrc(), getWeights(), getBias(), getScales(), getOffsets());
}

llvm::StringRef RowwiseQuantizedFullyConnectedInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Weights"; }
  if (idx == 3) { return "Bias"; }
  if (idx == 4) { return "Scales"; }
  if (idx == 5) { return "Offsets"; }
  llvm_unreachable("Invalid index");
}

void LocalResponseNormalizationGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " HalfWindowSize: " << getHalfWindowSize()
     << ", Alpha: " << getAlpha()
     << ", Beta: " << getBeta()
     << ", K: " << getK()
     << "}";
}

Instruction* LocalResponseNormalizationGradInst::clone() const {
  return new LocalResponseNormalizationGradInst(getName(), getDest(), getSrc(), getScale(), getDestGrad(), getSrcGrad(), getHalfWindowSize(), getAlpha(), getBeta(), getK());
}

llvm::StringRef LocalResponseNormalizationGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Scale"; }
  if (idx == 3) { return "DestGrad"; }
  if (idx == 4) { return "SrcGrad"; }
  llvm_unreachable("Invalid index");
}

void LocalResponseNormalizationInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " HalfWindowSize: " << getHalfWindowSize()
     << ", Alpha: " << getAlpha()
     << ", Beta: " << getBeta()
     << ", K: " << getK()
     << "}";
}

Instruction* LocalResponseNormalizationInst::clone() const {
  return new LocalResponseNormalizationInst(getName(), getDest(), getSrc(), getScale(), getHalfWindowSize(), getAlpha(), getBeta(), getK());
}

llvm::StringRef LocalResponseNormalizationInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Scale"; }
  llvm_unreachable("Invalid index");
}

void SoftMaxInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* SoftMaxInst::clone() const {
  return new SoftMaxInst(getName(), getDest(), getSrc());
}

llvm::StringRef SoftMaxInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void SoftMaxGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* SoftMaxGradInst::clone() const {
  return new SoftMaxGradInst(getName(), getOrigDest(), getOrigSrc(), getSelected(), getSrcGrad());
}

llvm::StringRef SoftMaxGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "OrigDest"; }
  if (idx == 1) { return "OrigSrc"; }
  if (idx == 2) { return "Selected"; }
  if (idx == 3) { return "SrcGrad"; }
  llvm_unreachable("Invalid index");
}

void CrossEntropyLossInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* CrossEntropyLossInst::clone() const {
  return new CrossEntropyLossInst(getName(), getP(), getLabels(), getCE());
}

llvm::StringRef CrossEntropyLossInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "P"; }
  if (idx == 1) { return "Labels"; }
  if (idx == 2) { return "CE"; }
  llvm_unreachable("Invalid index");
}

void CrossEntropyLossGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* CrossEntropyLossGradInst::clone() const {
  return new CrossEntropyLossGradInst(getName(), getCEGrad(), getP(), getLabels(), getPgrad(), getLabelsgrad());
}

llvm::StringRef CrossEntropyLossGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "CEGrad"; }
  if (idx == 1) { return "P"; }
  if (idx == 2) { return "Labels"; }
  if (idx == 3) { return "Pgrad"; }
  if (idx == 4) { return "Labelsgrad"; }
  llvm_unreachable("Invalid index");
}

void MatMulInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* MatMulInst::clone() const {
  return new MatMulInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef MatMulInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void BatchMatMulInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* BatchMatMulInst::clone() const {
  return new BatchMatMulInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef BatchMatMulInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void BatchedReduceAddInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Axis: " << getAxis()
     << "}";
}

Instruction* BatchedReduceAddInst::clone() const {
  return new BatchedReduceAddInst(getName(), getDest(), getBatch(), getAxis());
}

llvm::StringRef BatchedReduceAddInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Batch"; }
  llvm_unreachable("Invalid index");
}

void BatchedReduceMinInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Axes: " << getAxes()
     << "}";
}

Instruction* BatchedReduceMinInst::clone() const {
  return new BatchedReduceMinInst(getName(), getDest(), getBatch(), getAxes());
}

llvm::StringRef BatchedReduceMinInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Batch"; }
  llvm_unreachable("Invalid index");
}

void CumSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Exclusive: " << getExclusive()
     << ", Reverse: " << getReverse()
     << "}";
}

Instruction* CumSumInst::clone() const {
  return new CumSumInst(getName(), getDest(), getInput(), getExclusive(), getReverse());
}

llvm::StringRef CumSumInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Input"; }
  llvm_unreachable("Invalid index");
}

void LengthsSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* LengthsSumInst::clone() const {
  return new LengthsSumInst(getName(), getDest(), getData(), getLengths());
}

llvm::StringRef LengthsSumInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Data"; }
  if (idx == 2) { return "Lengths"; }
  llvm_unreachable("Invalid index");
}

void SparseLengthsSumGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " LengthsMode: " << getLengthsMode()
     << ", AvgLength: " << getAvgLength()
     << "}";
}

Instruction* SparseLengthsSumGradInst::clone() const {
  return new SparseLengthsSumGradInst(getName(), getData(), getIndices(), getLengths(), getDestGrad(), getDataGrad(), getLengthsMode(), getAvgLength());
}

llvm::StringRef SparseLengthsSumGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Data"; }
  if (idx == 1) { return "Indices"; }
  if (idx == 2) { return "Lengths"; }
  if (idx == 3) { return "DestGrad"; }
  if (idx == 4) { return "DataGrad"; }
  llvm_unreachable("Invalid index");
}

void SparseLengthsSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " LengthsMode: " << getLengthsMode()
     << ", AvgLength: " << getAvgLength()
     << "}";
}

Instruction* SparseLengthsSumInst::clone() const {
  return new SparseLengthsSumInst(getName(), getDest(), getData(), getIndices(), getLengths(), getLengthsMode(), getAvgLength());
}

llvm::StringRef SparseLengthsSumInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Data"; }
  if (idx == 2) { return "Indices"; }
  if (idx == 3) { return "Lengths"; }
  llvm_unreachable("Invalid index");
}

void SparseLengthsWeightedSumGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " LengthsMode: " << getLengthsMode()
     << ", AvgLength: " << getAvgLength()
     << "}";
}

Instruction* SparseLengthsWeightedSumGradInst::clone() const {
  return new SparseLengthsWeightedSumGradInst(getName(), getData(), getWeights(), getIndices(), getLengths(), getDestGrad(), getDataGrad(), getWeightsGrad(), getLengthsMode(), getAvgLength());
}

llvm::StringRef SparseLengthsWeightedSumGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Data"; }
  if (idx == 1) { return "Weights"; }
  if (idx == 2) { return "Indices"; }
  if (idx == 3) { return "Lengths"; }
  if (idx == 4) { return "DestGrad"; }
  if (idx == 5) { return "DataGrad"; }
  if (idx == 6) { return "WeightsGrad"; }
  llvm_unreachable("Invalid index");
}

void SparseLengthsWeightedSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " LengthsMode: " << getLengthsMode()
     << ", AvgLength: " << getAvgLength()
     << "}";
}

Instruction* SparseLengthsWeightedSumInst::clone() const {
  return new SparseLengthsWeightedSumInst(getName(), getDest(), getData(), getWeights(), getIndices(), getLengths(), getLengthsMode(), getAvgLength());
}

llvm::StringRef SparseLengthsWeightedSumInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Data"; }
  if (idx == 2) { return "Weights"; }
  if (idx == 3) { return "Indices"; }
  if (idx == 4) { return "Lengths"; }
  llvm_unreachable("Invalid index");
}

void EmbeddingBagInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " HasEndOffset: " << getHasEndOffset()
     << ", LengthsMode: " << getLengthsMode()
     << ", AvgLength: " << getAvgLength()
     << "}";
}

Instruction* EmbeddingBagInst::clone() const {
  return new EmbeddingBagInst(getName(), getDest(), getData(), getWeights(), getIndices(), getOffsets(), getHasEndOffset(), getLengthsMode(), getAvgLength());
}

llvm::StringRef EmbeddingBagInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Data"; }
  if (idx == 2) { return "Weights"; }
  if (idx == 3) { return "Indices"; }
  if (idx == 4) { return "Offsets"; }
  llvm_unreachable("Invalid index");
}

void RowwiseQuantizedSparseLengthsWeightedSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " UseFP16Accumulation: " << getUseFP16Accumulation()
     << ", LengthsMode: " << getLengthsMode()
     << ", AvgLength: " << getAvgLength()
     << "}";
}

Instruction* RowwiseQuantizedSparseLengthsWeightedSumInst::clone() const {
  return new RowwiseQuantizedSparseLengthsWeightedSumInst(getName(), getDest(), getData(), getScales(), getOffsets(), getWeights(), getIndices(), getLengths(), getUseFP16Accumulation(), getLengthsMode(), getAvgLength());
}

llvm::StringRef RowwiseQuantizedSparseLengthsWeightedSumInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Data"; }
  if (idx == 2) { return "Scales"; }
  if (idx == 3) { return "Offsets"; }
  if (idx == 4) { return "Weights"; }
  if (idx == 5) { return "Indices"; }
  if (idx == 6) { return "Lengths"; }
  llvm_unreachable("Invalid index");
}

void FusedRowwiseQuantizedSparseLengthsWeightedSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " UseFP16Accumulation: " << getUseFP16Accumulation()
     << ", LengthsMode: " << getLengthsMode()
     << ", AvgLength: " << getAvgLength()
     << "}";
}

Instruction* FusedRowwiseQuantizedSparseLengthsWeightedSumInst::clone() const {
  return new FusedRowwiseQuantizedSparseLengthsWeightedSumInst(getName(), getDest(), getData(), getWeights(), getIndices(), getLengths(), getUseFP16Accumulation(), getLengthsMode(), getAvgLength());
}

llvm::StringRef FusedRowwiseQuantizedSparseLengthsWeightedSumInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Data"; }
  if (idx == 2) { return "Weights"; }
  if (idx == 3) { return "Indices"; }
  if (idx == 4) { return "Lengths"; }
  llvm_unreachable("Invalid index");
}

void EmbeddingBagByteRowwiseOffsetsInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " UseFP16Accumulation: " << getUseFP16Accumulation()
     << ", HasEndOffset: " << getHasEndOffset()
     << ", LengthsMode: " << getLengthsMode()
     << ", AvgLength: " << getAvgLength()
     << "}";
}

Instruction* EmbeddingBagByteRowwiseOffsetsInst::clone() const {
  return new EmbeddingBagByteRowwiseOffsetsInst(getName(), getDest(), getData(), getWeights(), getIndices(), getOffsets(), getUseFP16Accumulation(), getHasEndOffset(), getLengthsMode(), getAvgLength());
}

llvm::StringRef EmbeddingBagByteRowwiseOffsetsInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Data"; }
  if (idx == 2) { return "Weights"; }
  if (idx == 3) { return "Indices"; }
  if (idx == 4) { return "Offsets"; }
  llvm_unreachable("Invalid index");
}

void LengthsToRangesInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* LengthsToRangesInst::clone() const {
  return new LengthsToRangesInst(getName(), getDest(), getLengths());
}

llvm::StringRef LengthsToRangesInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Lengths"; }
  llvm_unreachable("Invalid index");
}

void LengthsRangeFillInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* LengthsRangeFillInst::clone() const {
  return new LengthsRangeFillInst(getName(), getDest(), getLengths());
}

llvm::StringRef LengthsRangeFillInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Lengths"; }
  llvm_unreachable("Invalid index");
}

void SparseToDenseInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* SparseToDenseInst::clone() const {
  return new SparseToDenseInst(getName(), getDest(), getIndices(), getValues());
}

llvm::StringRef SparseToDenseInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Indices"; }
  if (idx == 2) { return "Values"; }
  llvm_unreachable("Invalid index");
}

void SparseToDenseMaskInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Mask: " << getMask()
     << "}";
}

Instruction* SparseToDenseMaskInst::clone() const {
  return new SparseToDenseMaskInst(getName(), getDest(), getIndices(), getValues(), getDefaultValue(), getLengths(), getMask());
}

llvm::StringRef SparseToDenseMaskInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Indices"; }
  if (idx == 2) { return "Values"; }
  if (idx == 3) { return "DefaultValue"; }
  if (idx == 4) { return "Lengths"; }
  llvm_unreachable("Invalid index");
}

void BatchedAddInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* BatchedAddInst::clone() const {
  return new BatchedAddInst(getName(), getDest(), getBatch(), getSlice());
}

llvm::StringRef BatchedAddInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Batch"; }
  if (idx == 2) { return "Slice"; }
  llvm_unreachable("Invalid index");
}

void ElementAddInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementAddInst::clone() const {
  return new ElementAddInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementAddInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementSubInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementSubInst::clone() const {
  return new ElementSubInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementSubInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementMulInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementMulInst::clone() const {
  return new ElementMulInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementMulInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementDivInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementDivInst::clone() const {
  return new ElementDivInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementDivInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementMaxInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementMaxInst::clone() const {
  return new ElementMaxInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementMaxInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementMinInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementMinInst::clone() const {
  return new ElementMinInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementMinInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementCmpEQInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementCmpEQInst::clone() const {
  return new ElementCmpEQInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementCmpEQInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementCmpNEQInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementCmpNEQInst::clone() const {
  return new ElementCmpNEQInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementCmpNEQInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementCmpLTInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementCmpLTInst::clone() const {
  return new ElementCmpLTInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementCmpLTInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementCmpLTEInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementCmpLTEInst::clone() const {
  return new ElementCmpLTEInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementCmpLTEInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementIsNaNInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementIsNaNInst::clone() const {
  return new ElementIsNaNInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementIsNaNInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementPowInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementPowInst::clone() const {
  return new ElementPowInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementPowInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementAndInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementAndInst::clone() const {
  return new ElementAndInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementAndInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementOrInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementOrInst::clone() const {
  return new ElementOrInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementOrInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementXorInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementXorInst::clone() const {
  return new ElementXorInst(getName(), getDest(), getLHS(), getRHS());
}

llvm::StringRef ElementXorInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "LHS"; }
  if (idx == 2) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ElementNotInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementNotInst::clone() const {
  return new ElementNotInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementNotInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementNegInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementNegInst::clone() const {
  return new ElementNegInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementNegInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementAbsInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementAbsInst::clone() const {
  return new ElementAbsInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementAbsInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementFloorInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementFloorInst::clone() const {
  return new ElementFloorInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementFloorInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementCeilInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementCeilInst::clone() const {
  return new ElementCeilInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementCeilInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementRoundInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementRoundInst::clone() const {
  return new ElementRoundInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementRoundInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementSqrtInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementSqrtInst::clone() const {
  return new ElementSqrtInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementSqrtInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementRsqrtInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementRsqrtInst::clone() const {
  return new ElementRsqrtInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementRsqrtInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementReciprocalInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementReciprocalInst::clone() const {
  return new ElementReciprocalInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementReciprocalInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementSinInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementSinInst::clone() const {
  return new ElementSinInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementSinInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementCosInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementCosInst::clone() const {
  return new ElementCosInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementCosInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementLogInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementLogInst::clone() const {
  return new ElementLogInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementLogInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementExpInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementExpInst::clone() const {
  return new ElementExpInst(getName(), getDest(), getSrc());
}

llvm::StringRef ElementExpInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ElementSelectInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ElementSelectInst::clone() const {
  return new ElementSelectInst(getName(), getDest(), getCond(), getLHS(), getRHS());
}

llvm::StringRef ElementSelectInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Cond"; }
  if (idx == 2) { return "LHS"; }
  if (idx == 3) { return "RHS"; }
  llvm_unreachable("Invalid index");
}

void ModuloInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Divisor: " << getDivisor()
     << ", SignFollowDivisor: " << getSignFollowDivisor()
     << "}";
}

Instruction* ModuloInst::clone() const {
  return new ModuloInst(getName(), getDest(), getSrc(), getDivisor(), getSignFollowDivisor());
}

llvm::StringRef ModuloInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void BatchedPairwiseDotProductInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " NumInputs: " << getNumInputs()
     << ", VectorSize: " << getVectorSize()
     << "}";
}

Instruction* BatchedPairwiseDotProductInst::clone() const {
  return new BatchedPairwiseDotProductInst(getName(), getDest(), getNumInputs(), getVectorSize());
}

llvm::StringRef BatchedPairwiseDotProductInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  llvm_unreachable("Invalid index");
}

void BatchedPairwiseDotProductGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " NumInputs: " << getNumInputs()
     << ", VectorSize: " << getVectorSize()
     << "}";
}

Instruction* BatchedPairwiseDotProductGradInst::clone() const {
  return new BatchedPairwiseDotProductGradInst(getName(), getDestGrad(), getNumInputs(), getVectorSize());
}

llvm::StringRef BatchedPairwiseDotProductGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "DestGrad"; }
  llvm_unreachable("Invalid index");
}

void ReluGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ReluGradInst::clone() const {
  return new ReluGradInst(getName(), getDest(), getDestGrad(), getSrcGrad());
}

llvm::StringRef ReluGradInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "DestGrad"; }
  if (idx == 2) { return "SrcGrad"; }
  llvm_unreachable("Invalid index");
}

void ReluInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ReluInst::clone() const {
  return new ReluInst(getName(), getDest(), getSrc());
}

llvm::StringRef ReluInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ClipInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Min: " << getMin()
     << ", Max: " << getMax()
     << "}";
}

Instruction* ClipInst::clone() const {
  return new ClipInst(getName(), getDest(), getSrc(), getMin(), getMax());
}

llvm::StringRef ClipInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void SigmoidInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* SigmoidInst::clone() const {
  return new SigmoidInst(getName(), getDest(), getSrc());
}

llvm::StringRef SigmoidInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void TanhInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* TanhInst::clone() const {
  return new TanhInst(getName(), getDest(), getSrc());
}

llvm::StringRef TanhInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void TransposeInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Shuffle: " << getShuffle()
     << "}";
}

Instruction* TransposeInst::clone() const {
  return new TransposeInst(getName(), getDest(), getSrc(), getShuffle());
}

llvm::StringRef TransposeInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void SplatInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Value: " << getValue()
     << "}";
}

Instruction* SplatInst::clone() const {
  return new SplatInst(getName(), getDest(), getValue());
}

llvm::StringRef SplatInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  llvm_unreachable("Invalid index");
}

void TouchInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* TouchInst::clone() const {
  return new TouchInst(getName(), getDest());
}

llvm::StringRef TouchInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  llvm_unreachable("Invalid index");
}

void InsertTensorInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Offsets: " << getOffsets()
     << ", Count: " << getCount()
     << ", Axis: " << getAxis()
     << "}";
}

Instruction* InsertTensorInst::clone() const {
  return new InsertTensorInst(getName(), getDest(), getSrc(), getOffsets(), getCount(), getAxis());
}

llvm::StringRef InsertTensorInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ExtractTensorInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Offsets: " << getOffsets()
     << "}";
}

Instruction* ExtractTensorInst::clone() const {
  return new ExtractTensorInst(getName(), getDest(), getSrc(), getOffsets());
}

llvm::StringRef ExtractTensorInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void GatherInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " BatchDims: " << getBatchDims()
     << "}";
}

Instruction* GatherInst::clone() const {
  return new GatherInst(getName(), getDest(), getData(), getIndices(), getBatchDims());
}

llvm::StringRef GatherInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Data"; }
  if (idx == 2) { return "Indices"; }
  llvm_unreachable("Invalid index");
}

void GatherRangesInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* GatherRangesInst::clone() const {
  return new GatherRangesInst(getName(), getOutput(), getLengths(), getData(), getRanges());
}

llvm::StringRef GatherRangesInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Output"; }
  if (idx == 1) { return "Lengths"; }
  if (idx == 2) { return "Data"; }
  if (idx == 3) { return "Ranges"; }
  llvm_unreachable("Invalid index");
}

void ScatterDataInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Cumulative: " << getCumulative()
     << "}";
}

Instruction* ScatterDataInst::clone() const {
  return new ScatterDataInst(getName(), getData(), getIndices(), getSlices(), getCumulative());
}

llvm::StringRef ScatterDataInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Data"; }
  if (idx == 1) { return "Indices"; }
  if (idx == 2) { return "Slices"; }
  llvm_unreachable("Invalid index");
}

void BatchOneHotInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* BatchOneHotInst::clone() const {
  return new BatchOneHotInst(getName(), getDest(), getData(), getLengths(), getValues());
}

llvm::StringRef BatchOneHotInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Data"; }
  if (idx == 2) { return "Lengths"; }
  if (idx == 3) { return "Values"; }
  llvm_unreachable("Invalid index");
}

void SpaceToDepthInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " BlockSize: " << getBlockSize()
     << "}";
}

Instruction* SpaceToDepthInst::clone() const {
  return new SpaceToDepthInst(getName(), getDest(), getSrc(), getBlockSize());
}

llvm::StringRef SpaceToDepthInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ResizeNearestInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Scale: " << getScale()
     << "}";
}

Instruction* ResizeNearestInst::clone() const {
  return new ResizeNearestInst(getName(), getDest(), getSrc(), getScale());
}

llvm::StringRef ResizeNearestInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void ResizeBilinearInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Scale: " << getScale()
     << "}";
}

Instruction* ResizeBilinearInst::clone() const {
  return new ResizeBilinearInst(getName(), getDest(), getSrc(), getScale());
}

llvm::StringRef ResizeBilinearInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void FlipInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Axis: " << getAxis()
     << "}";
}

Instruction* FlipInst::clone() const {
  return new FlipInst(getName(), getDest(), getSrc(), getAxis());
}

llvm::StringRef FlipInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void DebugPrintInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Format: " << getFormat()
     << ", FileName: " << getFileName()
     << "}";
}

Instruction* DebugPrintInst::clone() const {
  return new DebugPrintInst(getName(), getSrc(), getFormat(), getFileName());
}

llvm::StringRef DebugPrintInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void TraceEventInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Index: " << getIndex()
     << "}";
}

Instruction* TraceEventInst::clone() const {
  return new TraceEventInst(getName(), getData(), getIndex());
}

llvm::StringRef TraceEventInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Data"; }
  llvm_unreachable("Invalid index");
}

void QuantizationProfileInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* QuantizationProfileInst::clone() const {
  return new QuantizationProfileInst(getName(), getInputTensor(), getHistogram(), getComputationInfo());
}

llvm::StringRef QuantizationProfileInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "InputTensor"; }
  if (idx == 1) { return "Histogram"; }
  if (idx == 2) { return "ComputationInfo"; }
  llvm_unreachable("Invalid index");
}

void IntLookupTableInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* IntLookupTableInst::clone() const {
  return new IntLookupTableInst(getName(), getDest(), getSrc(), getMapping());
}

llvm::StringRef IntLookupTableInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Mapping"; }
  llvm_unreachable("Invalid index");
}

void QuantizeInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* QuantizeInst::clone() const {
  return new QuantizeInst(getName(), getDest(), getSrc());
}

llvm::StringRef QuantizeInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void DequantizeInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* DequantizeInst::clone() const {
  return new DequantizeInst(getName(), getDest(), getSrc());
}

llvm::StringRef DequantizeInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void RescaleQuantizedInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* RescaleQuantizedInst::clone() const {
  return new RescaleQuantizedInst(getName(), getDest(), getSrc());
}

llvm::StringRef RescaleQuantizedInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void TopKInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " K: " << getK()
     << "}";
}

Instruction* TopKInst::clone() const {
  return new TopKInst(getName(), getValues(), getIndices(), getInput(), getScratch(), getK());
}

llvm::StringRef TopKInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Values"; }
  if (idx == 1) { return "Indices"; }
  if (idx == 2) { return "Input"; }
  if (idx == 3) { return "Scratch"; }
  llvm_unreachable("Invalid index");
}

void ConvertToInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

Instruction* ConvertToInst::clone() const {
  return new ConvertToInst(getName(), getResult(), getInput());
}

llvm::StringRef ConvertToInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Result"; }
  if (idx == 1) { return "Input"; }
  llvm_unreachable("Invalid index");
}

void AudioSpectrogramInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " WindowSize: " << getWindowSize()
     << ", WindowStride: " << getWindowStride()
     << ", MagnitudeSquared: " << getMagnitudeSquared()
     << "}";
}

Instruction* AudioSpectrogramInst::clone() const {
  return new AudioSpectrogramInst(getName(), getSpectrogram(), getInput(), getWindow(), getTwiddleFactors(), getBitReverseIndices(), getComplexToRealWeights(), getWinOutScratch(), getFftOutScratch(), getWindowSize(), getWindowStride(), getMagnitudeSquared());
}

llvm::StringRef AudioSpectrogramInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Spectrogram"; }
  if (idx == 1) { return "Input"; }
  if (idx == 2) { return "Window"; }
  if (idx == 3) { return "TwiddleFactors"; }
  if (idx == 4) { return "BitReverseIndices"; }
  if (idx == 5) { return "ComplexToRealWeights"; }
  if (idx == 6) { return "WinOutScratch"; }
  if (idx == 7) { return "FftOutScratch"; }
  llvm_unreachable("Invalid index");
}

void MFCCInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " SampleRate: " << getSampleRate()
     << ", LowerFrequency: " << getLowerFrequency()
     << ", UpperFrequency: " << getUpperFrequency()
     << ", FilterBankCount: " << getFilterBankCount()
     << ", NumCoefficients: " << getNumCoefficients()
     << "}";
}

Instruction* MFCCInst::clone() const {
  return new MFCCInst(getName(), getCoefficients(), getSpectrogram(), getMelWeights(), getMelRanges(), getDctMat(), getScratch(), getSampleRate(), getLowerFrequency(), getUpperFrequency(), getFilterBankCount(), getNumCoefficients());
}

llvm::StringRef MFCCInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Coefficients"; }
  if (idx == 1) { return "Spectrogram"; }
  if (idx == 2) { return "MelWeights"; }
  if (idx == 3) { return "MelRanges"; }
  if (idx == 4) { return "DctMat"; }
  if (idx == 5) { return "Scratch"; }
  llvm_unreachable("Invalid index");
}

void NonMaxSuppressionInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " CenterPointBox: " << getCenterPointBox()
     << ", MaxOutputBoxesPerClass: " << getMaxOutputBoxesPerClass()
     << ", IouThreshold: " << getIouThreshold()
     << ", ScoreThreshold: " << getScoreThreshold()
     << ", IsTFVersion4: " << getIsTFVersion4()
     << "}";
}

Instruction* NonMaxSuppressionInst::clone() const {
  return new NonMaxSuppressionInst(getName(), getIndices(), getNumberOfSelectedIndices(), getBoxes(), getScores(), getCenterPointBox(), getMaxOutputBoxesPerClass(), getIouThreshold(), getScoreThreshold(), getIsTFVersion4());
}

llvm::StringRef NonMaxSuppressionInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Indices"; }
  if (idx == 1) { return "NumberOfSelectedIndices"; }
  if (idx == 2) { return "Boxes"; }
  if (idx == 3) { return "Scores"; }
  llvm_unreachable("Invalid index");
}

void CPUMaxSplatInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " SplatValue: " << getSplatValue()
     << "}";
}

Instruction* CPUMaxSplatInst::clone() const {
  return new CPUMaxSplatInst(getName(), getDest(), getSrc(), getSplatValue());
}

llvm::StringRef CPUMaxSplatInst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  llvm_unreachable("Invalid index");
}

void CPUConvDKKC8Inst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << "}";
}

Instruction* CPUConvDKKC8Inst::clone() const {
  return new CPUConvDKKC8Inst(getName(), getDest(), getSrc(), getFilter(), getBias(), getKernels(), getStrides(), getPads(), getGroup());
}

llvm::StringRef CPUConvDKKC8Inst::getOperandName(unsigned idx) const {
  if (idx == 0) { return "Dest"; }
  if (idx == 1) { return "Src"; }
  if (idx == 2) { return "Filter"; }
  if (idx == 3) { return "Bias"; }
  llvm_unreachable("Invalid index");
}

#include "glow/CPUSpecificInstrsVerification.h"
